% Calculates the brightness distortion values of each pixel in an image.
% Stores the result in a matrix of scalar values.
function alpha = brightnessDistortion(image, E, S);

% Set 0 to 1 as a minimum standard deviation to avoid divide by 0
tempS = S + (S == 0);

numeratorR = (image(:,:,1) .* E(:,:,1)) ./ (tempS(:,:,1) .^ 2);
numeratorG = (image(:,:,2) .* E(:,:,2)) ./ (tempS(:,:,2) .^ 2);
numeratorB = (image(:,:,3) .* E(:,:,3)) ./ (tempS(:,:,3) .^ 2);

denominatorR = (E(:,:,1) ./ tempS(:,:,1)) .^ 2;
denominatorG = (E(:,:,2) ./ tempS(:,:,2)) .^ 2;
denominatorB = (E(:,:,3) ./ tempS(:,:,3)) .^ 2;

numeratorSum = numeratorR + numeratorG + numeratorB;
denominatorSum = denominatorR + denominatorG + denominatorB;

% Avoid divide by 0 error
denominatorSum = denominatorSum + (denominatorSum == 0);

alpha = numeratorSum ./ denominatorSum;