function classifiedImage = classifyImage(image, E, S, A, B)
% Classifies each pixel in an image into four categories:
% 1. Original Background if it has both brightness and chromaticity similar
%    to those of the same pixel in the background model.
% 2. Shadow if it has similar chromaticity but lower brightness than those
%    of the same pixel in the background model.
% 3. Highlighted Background if it has similar chromaticity but higher
%    brightness that the background model.
% 4. Foreground Object if it has a different chromaticity from the
%    expected values of the background model.

global constOnes;
global TCD;
global TALO;
global TA1;
global TA2;
alpha = brightnessDistortion(image, E, S);
tempA = A + (A == 0);
normAlpha = (alpha - constOnes) ./ tempA;

% Default minimum bi 0.55
tempB = max(B, constOnes * 0.55);
normCD = colorDistortion(image, E, S, alpha) ./ tempB;

% Foreground = Red
FG = (normCD > TCD) | (normAlpha < TALO);
% Background = Green
BG = ~FG & ((normAlpha < TA1) & (normAlpha > TA2));
% Shadow = Blue
SH = ~FG & ~BG & (normAlpha < 0);
FG = FG & ~SH;
classifiedImage = FG;