% this function gets a grayscale image I as it's input and returns:
% joystickRegions - array of structs of connected components
% buttons - array of indices to the regions array indicating they are
% buttons, and each button's action number
% ellipses - array of indices to the regions array indicating they are
% ellipses


function [joystickRegions,buttons,ellipses,joystick ] = detectJoystickShapes(I)

%% step and line edge detection
BW = detectStepLineEdges(I);
%imshow(BW)
%return

%% remove small connected components
% numOfPixels = mean(BWsize)/10; %the groups minimum number of pixels
% BW = bwareaopen(BW,numOfPixels);


%% connected components
%drawboundingbox(BW);
compStats = regionprops(BW,'ConvexHull','ConvexArea','ConvexImage','BoundingBox');
if length(compStats)<2
    disp('Less than 2 connected components!')
    return
end


im
%find the remote (the ellipse with the largest convex area)
remote=0;
if isEllipse(compStats(1))
    remote = 1;
end
for i = 2:length(compStats)
    if remote==0 && isEllipse(compStats(i))
        remote = i;
    end
    if isEllipse(compStats(i)) && compStats(remote).ConvexArea<compStats(i).ConvexArea
        remote = i;
    end
end

% imshow(BW),hold on
b=1;
e=1;
buttons=[];
ellipses=[];
for i = 1:length(compStats)
   if ~regionTooSmall(compStats(i))
%        width=1;
       %hull=compStats(i).ConvexHull;
       if inRemoteOnly(i,remote,compStats)
           %width=4;
           buttons(1,b) = i; %#ok<AGROW>
           buttons(2,b) = 0; %#ok<AGROW> %action
           b=b+1;
       end
       
       if isEllipse(compStats(i))
           ellipses(e) = i; %#ok<AGROW>
           e=e+1;
       end
       
%        if isEllipse(compStats(i))
%            plot(hull(:,1)',hull(:,2)','LineWidth',width,'Color','red');
%        else
%            plot(hull(:,1)',hull(:,2)','LineWidth',width,'Color','green');
%        end
   end
end

buttons = findButtonsActions(buttons,compStats);

global initJoystickRegions;
initJoystickRegions = true;

joystick = remote;
joystickRegions = compStats;

end


function [BW] = detectStepLineEdges(img)

global higherHysteresisThreshold;
global lowerHysteresisThreshold;

%lowerHysteresisThreshold = 0.1;
%higherHysteresisThreshold = 0.15;

%lowerThreshold = 0.13;
%higherThreshold = 0.18;

[pc] = phasecong3(img);       % Calculate phase congruency.
BW =  hysthresh(pc, lowerHysteresisThreshold, higherHysteresisThreshold); % Hysteresis threshold

% zero out the image's frame (there are lines there sometimes for some reason)
frameSize = 2;
iSize = size(BW);
BW(1:frameSize,1:iSize(2)) = 0; %upper rows
BW(iSize(1)-frameSize+1:iSize(1),1:iSize(2)) = 0; %lower rows
BW(1:iSize(1),1:frameSize) = 0; %left columns
BW(1:iSize(1),iSize(2)-frameSize+1:iSize(2)) = 0; %right columns

end

function [isEllipse] = isEllipse(region)
    ellipsenessThreshold = 10; %fitError biggen than this means it is not an ellipse
    %the fit error is the average error for the shape's pixels

    hull=region.ConvexHull;
    
    
    xx=hull(:,1)';
    yy=hull(:,2)';
    

    a=(max(xx)-min(xx))/2;
    b=(max(yy)-min(yy))/2;
    centroid = [min(xx)+a,min(yy)+b];
    
    %allocate resources
    tmp = sum(yy>=centroid(2));
    upper_x = zeros(1,tmp);
    upper_hull_y = zeros(1,tmp);
    tmp = sum(yy<centroid(2));
    lower_x = zeros(1,tmp);
    lower_hull_y = zeros(1,tmp);
    
    %upper part
    j=1;
    for k=1:length(xx)
       if yy(k)>=centroid(2)
           %shift everything by the centroid to have a cannonical ellipse
           upper_x(j)=xx(k)-centroid(1);
           upper_hull_y(j)=yy(k)-centroid(2);
           j=j+1;
       end
    end
    upper_model_y=sqrt((1 - ((upper_x.^2)./(a^2))).*b^2);
    upperError = abs(upper_hull_y - upper_model_y); %sum of differencees

    %lower part
    j=1;
    for k=1:length(xx)
       if yy(k)<centroid(2)
           %shift everything by the centroid to have a cannonical ellipse
           lower_x(j)=xx(k)-centroid(1);
           lower_hull_y(j)=yy(k)-centroid(2);
           j=j+1;
       end
    end
    lower_model_y=-sqrt((1 - ((lower_x.^2)./(a^2))).*b^2);
    lowerError = abs(lower_hull_y - lower_model_y); %sum of differencees
    
    % let the fit error be the average error for the shape's pixels
    fitError = (sum(upperError) + sum(lowerError))/length(xx);
    
    if fitError <= ellipsenessThreshold
        isEllipse = true;
    else
        isEllipse = false;
    end
end

function [tooSmall] = regionTooSmall(region)
    area = region.ConvexArea;
    threshold = 500;
    
    if area < threshold
        tooSmall=true;
    else
        tooSmall=false;
    end
end


function [inRemoteOnly] = inRemoteOnly(region,remote,regions)
    hull=regions(region).ConvexHull;
    regionPixelC=floor(hull(1,1));
    regionPixelR=floor(hull(1,2));
    
    
    
    
    if isPixelInRegion(regionPixelR,regionPixelC,regions(remote))
        inRemoteOnly=true;
    else
        inRemoteOnly=false;
        return;
    end
    if region==remote
        inRemoteOnly=false;
        return;
    end
    for i=1:length(regions)
       if i~=remote && i~=region && isPixelInRegion(regionPixelR,regionPixelC,regions(i)) && isEllipse(regions(i))
            inRemoteOnly=false;
       end
    end
end


function [buttons] = findButtonsActions(buttons,regions)
    
    numOfButtons = size(buttons);
    numOfButtons = numOfButtons(2);

    
    if numOfButtons~=5
        disp('THERE ARENT EXACTLY 5 BUTTONS')
    end

    % get buttons centers
    bCenters = zeros(2,numOfButtons);
    for i=1:numOfButtons
        hull=regions(buttons(1,i)).ConvexHull;

        xx=hull(:,1)';
        yy=hull(:,2)';

        a=(max(xx)-min(xx))/2;
        b=(max(yy)-min(yy))/2;
        %centroid = [min(xx)+a,min(yy)+b];
        bCenters(1,i) = min(xx)+a;
        bCenters(2,i) = min(yy)+b;
    end
    
    %remote kinds: 1=arrows, 2=hello world
    remoteKind=2;
    y_threshold = 30;
    for i=2:numOfButtons
        %dist = abs(bCenters(2,i)-bCenters(2,1))
        if abs(bCenters(2,i)-bCenters(2,1)) > y_threshold
            remoteKind=1; %arrows
        end
    end
    remoteKind
    %%tt=size(buttons);
    %tt(2)
    
    if remoteKind==1 %arrows
        %leftest is right_arrow (action 1)
        leftest = 1;
        for i=2:numOfButtons
            if bCenters(1,i) < bCenters(1,leftest)
                leftest = i;
            end
        end
        %rightest is left_arrow (action 2)
        rightest = 1;
        for i=2:numOfButtons
            if bCenters(1,i) > bCenters(1,rightest)
                rightest = i;
            end
        end
        %highest is down_arrow (action 3)
        highest = 1;
        for i=2:numOfButtons
            if bCenters(2,i) < bCenters(2,highest)
                highest = i;
            end
        end
        %lowest is up_arrow (action 4)
        lowest = 1;
        for i=2:numOfButtons
            if bCenters(2,i) > bCenters(2,lowest)
                lowest = i;
            end
        end
        %the one left is space (action 5)
        space = 0;
        for i=1:numOfButtons
            if i~=lowest && i~=highest && i~=rightest && i~=leftest
                space = i;
                break;
            end
        end
    
        if numOfButtons>=5
            buttons(2,leftest) = 1;
            buttons(2,rightest) = 2;
            buttons(2,highest) = 3;
            buttons(2,lowest) = 4;
            buttons(2,space) = 5;
        end
    else %hello world
        % button order from left to right:
        % actions - 6 to 10, form left to right (of the camera)
        
        %bCenters(1,leftest)
        xCenters = bCenters(1,:);
        s = sort(xCenters);
        
        for i=1:length(s)
            if i>5
                break;
            end
            
            %go over the 5 buttons with lowest x in order
            j = s(i); %find the button with x=j and set it's action to i
            if numOfButtons>=5
                for k=1:length(xCenters)
                    if j==xCenters(k)
                        buttons(2,k)=i+5;
                    end
                end
            end
        end 
    end
end
