% this function gets:
% boundary - a hand's boundary (as a set of points recieved from
% bwtraceboundary).
% k - we check the angle between a pixel and 2 pixels that are k-points
% distant from it.
% angleThreshold - points with angles lower than this are considered as
% finger tips
%
% returns a set of points (row,col) of finger tips

function [fingerTips] = fingerTipsFromBoundary(boundary,k,angleThreshold)

    %k=20;
    %angleThreshold=100;


    angleDegrees = zeros(1,length(boundary));
    for i=1:length(boundary)
        i1=mod((i-1)-k,length(boundary)) + 1;
        i2=mod((i-1)+k,length(boundary)) + 1;

        pc = [boundary(i,2),boundary(i,1)];
        p1 = [boundary(i1,2),boundary(i1,1)];
        p2 = [boundary(i2,2),boundary(i2,1)];
        v1 = [p1(1)-pc(1),p1(2)-pc(2)];
        v1 = v1/norm(v1); %normalize
        v2 = [p2(1)-pc(1),p2(2)-pc(2)];
        v2 = v2/norm(v2); %normalize
        angle = acos(v1*v2');
        angleDegrees(i) = angle*(180/pi);

        %if the vector is not pointing downwards than say it has a big angle (to discard it)
        if ~(pc(2)>=p1(2) && pc(2)>=p2(2))
            angleDegrees(i) = 180;
        end
    end
    % NOTICE - for some points an angle can't be calculated because the points
    % dont create 2 vectors. for these points the value is NaN. (an array cell
    % can be compared to NaN using isnan(num) and maybe if num==NaN works
    % as well.

    fingerTipCandidate = angleDegrees<angleThreshold; %NaNs will be 0

    chaining=false;
    chain_len=0;
    chain_sum=0;
    chain_start=0;
    fingerTipsIndex = 1;
    handlingStartChain = false;
    startChainLength = 0;
    for i=1:length(fingerTipCandidate)
        if fingerTipCandidate(i)==1
            if i==1
                handlingStartChain = true;
            end

            chaining = true;
            chain_sum = chain_sum + i;
            chain_len = chain_len + 1;
            if chain_start==0
                chain_start = i;
            end
        else
            if chaining == true
                fingerTips(fingerTipsIndex,1) = boundary(floor(chain_sum/chain_len),1);
                fingerTips(fingerTipsIndex,2) = boundary(floor(chain_sum/chain_len),2);
                fingerTipsIndex = fingerTipsIndex + 1;

                if handlingStartChain==true
                    startChainLength = chain_len;
                    handlingStartChain = false;
                end
                chaining=false;
                chain_len=0;
                chain_sum=0;
                chain_start=0;
            end
        end
    end
    if i==length(fingerTipCandidate) && chaining==true %the array doesnt end with 0 so we have to take care of it explicitly (and add it to the beginning if the first array cell is also 1)
        if startChainLength==0 %no chain at the start
            fingerTips(fingerTipsIndex,1) = boundary(floor(chain_sum/chain_len),1);
            fingerTips(fingerTipsIndex,2) = boundary(floor(chain_sum/chain_len),2);
        else %we have a chain at start, need to find the middle
            endChainLength = chain_len;
            chain_len = endChainLength + startChainLength;
            chain_mid = ceil(chain_len/2); %ceil here is to handle the case when chain_len is odd
            if endChainLength>=chain_mid
                fingerTips(1,1) = boundary(chain_start+chain_mid-1,1);
                fingerTips(1,2) = boundary(chain_start+chain_mid-1,2);
            else

                fingerTips(1,1) = boundary(chain_mid-endChainLength,1);
                fingerTips(1,2) = boundary(chain_mid-endChainLength,2);
            end
        end
    end
    if fingerTipsIndex==1 %no fingertips found
        fingerTips=[];
    end


end